package digital.laboratory.platform.identifybook.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import digital.laboratory.platform.common.core.constant.CommonConstants;
import digital.laboratory.platform.common.core.exception.CheckedException;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.identifybook.config.ApiPathProperties;
import digital.laboratory.platform.identifybook.dto.EntrustmentDTO;
import digital.laboratory.platform.identifybook.emums.AnnexTypeEnum;
import digital.laboratory.platform.identifybook.emums.ReceiveWayEnum;
import digital.laboratory.platform.identifybook.entity.Entrustment;
import digital.laboratory.platform.identifybook.entity.IdentifyBookApprove;
import digital.laboratory.platform.identifybook.service.IEntrustmentService;
import digital.laboratory.platform.identifybook.service.IIdentifyBookApproveService;
import digital.laboratory.platform.identifybook.service.PushDataToLabsCareService;
import digital.laboratory.platform.inspetion.api.entity.EntrustInfo;
import digital.laboratory.platform.inspetion.api.entity.IdentificationBookDTO;
import digital.laboratory.platform.inspetion.api.entity.SampleInfo;
import digital.laboratory.platform.inspetion.api.enums.BusinessType;
import digital.laboratory.platform.inspetion.api.feign.RemoteTestToIdentifyService;
import digital.laboratory.platform.othersys.utils.HttpsUtils;
import digital.laboratory.platform.sys.feign.RemoteEntrustmentService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.*;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestClientException;

import javax.annotation.Resource;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author ChenJiangBao
 * @version 1.0
 * @description: 推送数据到labscare平台接口
 * @date 2025/3/31 9:31
 */
@Slf4j
@Service
public class PushDataToLabsCareServiceImpl implements PushDataToLabsCareService {

    @Resource
    private OssFile ossFile;

    @Resource
    private IEntrustmentService entrustService;

    @Resource
    private IIdentifyBookApproveService identifyBookApproveService;

    @Resource
    private RemoteEntrustmentService remoteEntrustmentService;

    @Resource
    private RemoteTestToIdentifyService remoteTestToIdentifyService;

    @Resource
    private ApiPathProperties apiPathProperties;

    // "#NULL#" 作为空数据的占位符, 对于系统中没有的信息以占位符代替
    public final static String NULL_PLACEHOLDER = "#NULL#";

    // 鉴定报告审批表标识
    private final static String APPRAISAL_REPORT_APPROVAL_FORM = "AppraisalReportApprovalForm";

    // 定性报告标识
    private final static String QUALITATIVE_REPORT = "QualitativeReport";

    // 归档报告信息标识
    private final static String ARCHIVE_REPORT_INFO = "ArchiveReportInfo";

    /**
     * 做一个定时推送，对于推送失败的委托进行重新推送， 每天凌晨3点推送
     */
//    @Scheduled(cron = "30 * * * * ?") // 测试
    @Scheduled(cron = "0 0 3 * * ?")  // 每天凌晨 3 点执行
    public void timingPushDataToLabsCare() {
        log.info("定时推送数据到LabsCare......");
        // 查询检验鉴定推送失败的委托
        List<Entrustment> entrustList = entrustService.list(
                Wrappers.<Entrustment>lambdaQuery()
                        .eq(Entrustment::getBusinessType, BusinessType.NPS_CASE.getBusinessType()) // 只有常规毒品的才需要处理
                        .and(wrapper -> wrapper
                                .eq(Entrustment::getPushFlag, "")
                                .or()
                                .isNull(Entrustment::getPushFlag)
                                .or()
                                .like(Entrustment::getPushFlag, "false")
                        )
        );
        if (CollUtil.isEmpty(entrustList)) {
            return;
        }

        for (Entrustment entrustment : entrustList) {
            String pushFlag = entrustment.getPushFlag();
            if (StrUtil.isBlank(pushFlag)) {
//                pushAppraisalReportApprovalFormData(entrustment.getId());
                pushQualitativeReportData(entrustment.getId());
            } else {
                List<String> flagList = StrUtil.split(pushFlag, StrUtil.COMMA).stream().filter(str -> str.contains("false")).collect(Collectors.toList());
                for (String flag : flagList) {
                    if (flag.contains(APPRAISAL_REPORT_APPROVAL_FORM)) {
//                        pushAppraisalReportApprovalFormData(entrustment.getId());
                    } else {
                        pushQualitativeReportData(entrustment.getId());
                    }
                }
            }
        }
    }

    /**
     * 推送定性报告数据
     *
     * @param entrustId 委托id
     */
    @Override
    public void pushQualitativeReportData(String entrustId) {
        // 根据委托id获取鉴定审批信息
        EntrustmentDTO entrustmentDTO = entrustService.getEntrustmentDTO(entrustId);

        if (!entrustmentDTO.getBusinessType().toString().equals(BusinessType.NPS_CASE.getBusinessType())) {
            return;
        }

        // 构建jsonObject 参数
        JSONObject jsonObject = buildQualitativeReportJsonPayload(entrustmentDTO);

        String successFlag = QUALITATIVE_REPORT + ":true";
        String failureFlag = QUALITATIVE_REPORT + ":false";
        try {
            ResponseEntity<String> response = exchangeRemoteApi(jsonObject, apiPathProperties.getQualitativeReport());

            if (response.getStatusCode().is2xxSuccessful() && response.getBody().contains("\"status\":200")) {
                updatePushFlag(entrustmentDTO, QUALITATIVE_REPORT, successFlag);
                log.info("推送定性报告数据成功, 受理编号: {}, 响应: {}", entrustmentDTO.getAcceptNo(), response.getBody());
            } else {
                updatePushFlag(entrustmentDTO, QUALITATIVE_REPORT, failureFlag);
                log.warn("推送定性报告数据失败, 受理编号: {}, 状态码: {}, 响应: {}",
                        entrustmentDTO.getAcceptNo(), response.getStatusCode(), response.getBody());
            }
        } catch (RestClientException e) {
            log.error("推送定性报告数据到 LabsCare 失败, 受理编号: {}", entrustmentDTO.getAcceptNo(), e);
        }
    }

    /**
     * 推送鉴定报告审批表数据
     *
     * @param entrustId 委托id
     */
    @Override
    public void pushAppraisalReportApprovalFormData(String entrustId) {
        // 根据委托id获取鉴定审批信息
        IdentifyBookApprove identifyBookApprove = identifyBookApproveService.getOne(Wrappers.<IdentifyBookApprove>lambdaQuery()
                .eq(IdentifyBookApprove::getRelevanceBusinessId, entrustId));
        EntrustmentDTO entrustmentDTO = entrustService.getEntrustmentDTO(entrustId);

        // 构建jsonObject 参数
        JSONObject jsonObject = buildAppraisalReportApprovalFormJsonPayload(identifyBookApprove, entrustmentDTO);

        String successFlag = APPRAISAL_REPORT_APPROVAL_FORM + ":true";
        String failureFlag = APPRAISAL_REPORT_APPROVAL_FORM + ":false";
        try {
            ResponseEntity<String> response = exchangeRemoteApi(jsonObject, apiPathProperties.getAppraisalReportApprovalForm());

            if (response.getStatusCode().is2xxSuccessful() && response.getBody().contains("\"status\":200")) {
                updatePushFlag(entrustmentDTO, APPRAISAL_REPORT_APPROVAL_FORM, successFlag);
                log.info("推送鉴定报告审批表报告数据成功, 受理编号: {}, 响应: {}", entrustmentDTO.getAcceptNo(), response.getBody());
            } else {
                updatePushFlag(entrustmentDTO, APPRAISAL_REPORT_APPROVAL_FORM, failureFlag);
                log.warn("推送鉴定报告审批表报告数据失败, 受理编号: {}, 状态码: {}, 响应: {}",
                        entrustmentDTO.getAcceptNo(), response.getStatusCode(), response.getBody());
            }
        } catch (RestClientException e) {
            log.error("推送鉴定报告审批表报告数据到 LabsCare 失败, 受理编号: {}", entrustmentDTO.getAcceptNo(), e);
        }
    }

    /**
     * 推送归档报告信息数据
     *
     * @param entrustId 委托id
     */
    @Override
    public void pushArchiveReportInfoData(String entrustId) {

        // 获取数据
        EntrustmentDTO entrustmentDTO = entrustService.getEntrustmentDTO(entrustId);
        // 构建jsonObject 参数
        JSONObject jsonObject = buildArchiveReportInfoJsonPayload(entrustmentDTO);

        String successFlag = ARCHIVE_REPORT_INFO + ":true";
        String failureFlag = ARCHIVE_REPORT_INFO + ":false";
        try {
            ResponseEntity<String> response = exchangeRemoteApi(jsonObject, apiPathProperties.getAppraisalReportApprovalForm());

            if (response.getStatusCode().is2xxSuccessful() && response.getBody().contains("\"status\":200")) {
                updatePushFlag(entrustmentDTO, ARCHIVE_REPORT_INFO, successFlag);
                log.info("推送归档报告信息数据成功, 受理编号: {}, 响应: {}", entrustmentDTO.getAcceptNo(), response.getBody());
            } else {
                updatePushFlag(entrustmentDTO, ARCHIVE_REPORT_INFO, failureFlag);
                log.warn("推送归档报告信息数据失败, 受理编号: {}, 状态码: {}, 响应: {}",
                        entrustmentDTO.getAcceptNo(), response.getStatusCode(), response.getBody());
            }
        } catch (RestClientException e) {
            log.error("推送归档报告信息数据到 LabsCare 失败, 受理编号: {}", entrustmentDTO.getAcceptNo(), e);
        }
    }

    /**
     * 构建推送定性报告的json数据
     *
     * @param entrustmentDTO 委托信息
     * @return
     */
    private JSONObject buildQualitativeReportJsonPayload(EntrustmentDTO entrustmentDTO) {
        //获取到制作文书的实验数据
        R<IdentificationBookDTO> identifyBookData = remoteTestToIdentifyService.getIdentifyBookDataByBusinessId(entrustmentDTO.getId());
        IdentificationBookDTO identificationBookDTO = identifyBookData.getData();
        // 对受理编号进行分隔
        EntrustInfo entrustInfo = identificationBookDTO.getEntrustInfo();
        List<String> splitAcceptNo = StrUtil.split(entrustInfo.getAcceptNo(), "-");
        // 根据委托id获取鉴定审批信息
        IdentifyBookApprove identifyBookApprove = identifyBookApproveService.getOne(Wrappers.<IdentifyBookApprove>lambdaQuery()
                .eq(IdentifyBookApprove::getRelevanceBusinessId, entrustmentDTO.getId()));

        // 创建 SimpleDateFormat

        // 封装推送定性报告数据
        JSONObject jsonObject = new JSONObject();
        jsonObject.set("jcxz", buildMaterialCharacterDesc(identificationBookDTO.getSampleInfoList())); // 检材性状描述
        jsonObject.set("jiandingwenshuhao", String.format("[%s]%s号", splitAcceptNo.get(0), splitAcceptNo.get(1))); // 鉴定文书号
        jsonObject.set("jdyq", entrustInfo.getEntrustRequirement()); // 鉴定要求
        jsonObject.set("jyaq", entrustInfo.getCaseBrief()); // 简要案情
        jsonObject.set("jyy1qm", entrustmentDTO.getAppraisalUserName()); // 检测人
        jsonObject.set("jyyr2qm", entrustmentDTO.getAppraisalUserName()); // 检测人
        jsonObject.set("jyy1zc", NULL_PLACEHOLDER); // 检验人1专业技术资格或职称
        jsonObject.set("jyy2zc", NULL_PLACEHOLDER); // 检验人2专业技术资格或职称
        jsonObject.set("qfrq", NULL_PLACEHOLDER);
        jsonObject.set("qrcode", NULL_PLACEHOLDER); // 二维码地址
        jsonObject.set("sjr1List", CollUtil.newArrayList(entrustInfo.getDeliver1Name(), entrustInfo.getDeliver2Name())); // 送检人
        jsonObject.set("slrq", entrustInfo.getAcceptDate().format(DateTimeFormatter.ofPattern("yyyy年MM月dd日")));
        jsonObject.set("wtdw", entrustInfo.getEntrustDepartment()); // 委托单位
        jsonObject.set("zrqm", NULL_PLACEHOLDER); // 授权签字人
        jsonObject.set("zrzc", NULL_PLACEHOLDER); // 授权签字人专业技术资格或职称

        // 定性检验报告样品 检测信息
        JSONObject inspectDataInfo = new JSONObject();
        inspectDataInfo.set("dxff", identificationBookDTO.getTestMethod());
        inspectDataInfo.set("jysj", identificationBookDTO.getTestStartDate());
        inspectDataInfo.set("outStr", identificationBookDTO.getTestResult());
        JSONArray gauging = new JSONArray();
        gauging.add(inspectDataInfo);

        // 定性检验报告样品信息
        JSONObject sampleJson = new JSONObject();
        sampleJson.put("gauging", gauging);
        JSONArray sampleArrayJSON = new JSONArray();
        sampleArrayJSON.add(sampleJson);

        jsonObject.set("sample", sampleArrayJSON);
        return jsonObject;
    }

    /**
     * 构建推送鉴定审批表的json数据
     *
     * @param identifyBookApprove 鉴定文书审批表
     * @param entrustmentDTO      委托信息
     * @return
     */
    private JSONObject buildAppraisalReportApprovalFormJsonPayload(IdentifyBookApprove identifyBookApprove, EntrustmentDTO entrustmentDTO) {
        // 获取数据
        // 对受理编号进行分隔
        List<String> splitAcceptNo = StrUtil.split(entrustmentDTO.getAcceptNo(), "-");
        // 创建 SimpleDateFormat
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日");
        // 相关材料
        List<String> annexTypeList = StrUtil.split(identifyBookApprove.getAnnexType(), ",");
        String annexTypeStr = annexTypeList.stream().map(annexType -> AnnexTypeEnum.fromCode(annexType).getDesc()).collect(Collectors.joining(","));

        // 封装json数据
        JSONObject jsonObject = new JSONObject();
        jsonObject.set("bzyj", identifyBookApprove.getProcessMessage()); // 授权签字人意见
        jsonObject.set("bzysqqzr", identifyBookApprove.getProcessUserName()); // 授权签字人
        jsonObject.set("jdwsh", String.format("[%s]%s号", splitAcceptNo.get(0), splitAcceptNo.get(1))); // 鉴定文书号
        jsonObject.set("jwbgfws", ReceiveWayEnum.fromCode(entrustmentDTO.getReportReceiveMode()).getDesc() + ":" + identifyBookApprove.getInspectionReportNumber());
        jsonObject.set("jyy1qm", entrustmentDTO.getAppraisalUserName()); // 审批人
        jsonObject.set("qfrq", sdf.format(identifyBookApprove.getApproveTime())); // 格式化日期, 签发日期
        jsonObject.set("sjdw", entrustmentDTO.getClientOrgName()); // 送检单位
        jsonObject.set("sqqzrshrq", sdf.format(identifyBookApprove.getProcessTime())); // 	授权人审核日期
        jsonObject.set("tjsprq", sdf.format(identifyBookApprove.getProcessTime())); // 提交审批时间
        jsonObject.set("xgjc", annexTypeStr); // 相关材料
        jsonObject.set("zrqfyj", identifyBookApprove.getApproveMessage()); // 实验室负责人意见
        jsonObject.set("zrqm", identifyBookApprove.getApproveUserName()); // 实验室负责人签字
        jsonObject.set("zzrdxm", NULL_PLACEHOLDER); // 是否资质认定项目
        return jsonObject;
    }

    /**
     * 构建推送归档报告信息报告的json数据
     *
     * @return
     */
    private JSONObject buildArchiveReportInfoJsonPayload(EntrustmentDTO entrustmentDTO) {
        String entrustId = entrustmentDTO.getId();
        // 1 检验记录
        R<JSONObject> inspectRecordR = remoteTestToIdentifyService.inspectRecord(entrustId);
        if (inspectRecordR.getCode() == CommonConstants.FAIL) {
            throw new CheckedException("检验记录数据获取失败！");
        }
        JSONObject inspectRecordData = inspectRecordR.getData();

        // 2 获取委托书数据
        R<JSONObject> entrustLetterR = remoteEntrustmentService.entrustLetter(entrustmentDTO.getSynEntrustId());
        if (entrustLetterR.getCode() == CommonConstants.FAIL) {
            throw new CheckedException("获取委托书数据失败！");
        }
        JSONObject entrustLetterData = entrustLetterR.getData();

        // 3 获取鉴定事项确认书数据
        R<JSONObject> itemConfirmLetterR = remoteEntrustmentService.itemConfirmLetter(entrustmentDTO.getSynEntrustId());
        if (itemConfirmLetterR.getCode() == CommonConstants.FAIL) {
            throw new CheckedException("获取委托书数据失败！");
        }
        JSONObject itemConfirmLetterData = itemConfirmLetterR.getData();

        // 根据委托id获取鉴定审批信息
        IdentifyBookApprove identifyBookApprove = identifyBookApproveService.getOne(Wrappers.<IdentifyBookApprove>lambdaQuery()
                .eq(IdentifyBookApprove::getRelevanceBusinessId, entrustId));

        // 封装推送归档报告信息
        JSONObject jsonObject = new JSONObject();
        if (entrustmentDTO.getBusinessType().equals(BusinessType.BOINT_CASE.getBusinessType())) {
            jsonObject.set("shenWuJianCaiDingXingJiLuReportDto", inspectRecordData);
        } else {
            jsonObject.set("puTongJianCaiDingXingJiLuFeiReportDto", inspectRecordData);
        }
        jsonObject.set("jianDingWeiTuoShuReportDto", entrustLetterData);
        jsonObject.set("jianDingShiXiangQueRenShuReportDto", itemConfirmLetterData);
        jsonObject.set("jianDingShenPiReportDto", buildAppraisalReportApprovalFormJsonPayload(identifyBookApprove, entrustmentDTO));
        jsonObject.set("dingXingReportDto", buildQualitativeReportJsonPayload(entrustmentDTO));
        String path = "document/accept" + "/" + entrustId + "/materialPhoto/materialallpic.docx";
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();

        try {
            ossFile.fileGet(path, outputStream); // 读取 .docx 文件
            String base64String = Base64.getEncoder().encodeToString(outputStream.toByteArray()); // 转 Base64
            jsonObject.set("jctp", base64String); // 存入 JSON
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            try {
                outputStream.close(); // 释放资源
            } catch (IOException ignored) {
            }
        }
        return jsonObject;
    }


    /**
     * 通过POST请求与远程API进行交互
     *
     * @param jsonObject 包含请求数据的JSONObject对象
     * @param url        远程API的URL
     * @return ResponseEntity<String> 包含远程API响应数据的ResponseEntity对象
     */
    private ResponseEntity<String> exchangeRemoteApi(JSONObject jsonObject, String url) {
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);  // 设置为 application/json
        HttpEntity<JSONObject> requestEntity = new HttpEntity<>(jsonObject, headers);
        ResponseEntity<String> response = HttpsUtils
                .genRestTemplate()
                .exchange(
                        url,
                        HttpMethod.POST,
                        requestEntity,
                        String.class
                );
        return response;
    }

    /**
     * 更新推送标志
     *
     * @param entrust  委托对象
     * @param pushType 推送类型 EntrustLetter 委托书 | ItemConfirmLetter 事项确认书
     * @param newFlag  新的推送标志
     */
    private void updatePushFlag(Entrustment entrust, String pushType, String newFlag) {
        List<String> flagList = Optional.ofNullable(StrUtil.split(entrust.getPushFlag(), StrUtil.COMMA))
                .orElse(new ArrayList<>());

        // 判断是否已有 指定的推送类型，如果有就替换，否则添加
        boolean exists = flagList.stream().anyMatch(flag -> flag.contains(pushType));
        if (exists) {
            flagList.replaceAll(flag -> flag.contains(pushType) ? newFlag : flag);
        } else {
            flagList.add(newFlag);
        }
        // 更新标识, 防止定时任务那里连续推送数据时推送标识更新出错
        entrust.setPushFlag(String.join(",", flagList));
        // 更新数据库
        entrustService.update(Wrappers.<Entrustment>lambdaUpdate()
                .eq(Entrustment::getId, entrust.getId())
                .set(Entrustment::getPushFlag, String.join(",", flagList)));
    }

    public String buildMaterialCharacterDesc(List<SampleInfo> sampleInfoList) {
        if (CollUtil.isEmpty(sampleInfoList)) {
            return StrUtil.EMPTY;
        }

        // 按尾号排序（提取编号尾号进行比较）
        sampleInfoList.sort(Comparator.comparing(sample -> extractTailNumber(sample.getAcceptNo())));

        List<String> descriptions = new ArrayList<>();
        int start = -1, prev = -1;
        String prevForm = null;
        List<String> mergedOrderNos = new ArrayList<>();

        // 遍历排序后的检材编号，合并相邻且性状相同的编号
        for (SampleInfo sample : sampleInfoList) {
            int current = extractTailNumber(sample.getAcceptNo()); // 提取尾号
            String form = sample.getForm();

            if (start == -1) { // 初始化
                start = prev = current;
                prevForm = form;
            } else if (current == prev + 1 && form.equals(prevForm)) {
                // 只有连续尾号 + 相同性状才能合并
                prev = current;
            } else {
                // 遇到新性状或不连续尾号，存储前一段描述
                mergedOrderNos.add(formatRange(start, prev) + "检材为" + prevForm);
                start = prev = current;
                prevForm = form;
            }
        }
        // 处理最后一组数据
        mergedOrderNos.add(formatRange(start, prev) + "检材为" + prevForm);

        // **如果只有一个检材，省略编号**
        if (mergedOrderNos.size() == 1 && sampleInfoList.size() == 1) {
            return "检材为" + prevForm + "。";
        }

        return String.join("；", mergedOrderNos) + "。";
    }

    /**
     * 生成编号范围的字符串
     *
     * @param start 起始编号
     * @param end   结束编号
     * @return 生成的范围字符串，例如 "1号至3号" 或 "5号"
     */
    private String formatRange(int start, int end) {
        return (start == end) ? start + "号" : start + "号至" + end + "号";
    }

    /**
     * 提取检材编号的尾号（即最后一个 `-` 之后的数值）
     *
     * @param orderNo 检材编号（格式：2025-89-13）
     * @return 尾号（示例：返回 13）
     */
    private int extractTailNumber(String orderNo) {
        String[] parts = orderNo.split("-");
        return (parts.length > 0) ? Integer.parseInt(parts[parts.length - 1]) : 0;
    }
}
